from typing import List

import numpy as np
import pandas as pd
import scipy.io

from graphysio.plotwidgets.curves import CurveItem


def curves_to_matlab(
    curves: List[CurveItem], filepath: str, index_label: str = 'timens'
) -> None:
    sers = [c.series for c in curves]
    data = pd.concat(sers, axis=1).sort_index()
    data['timens'] = data.index.astype(np.int64)
    scipy.io.savemat(filepath, {'data': data.to_dict("list")})



from collections import namedtuple
from functools import partial

import numpy as np
import pandas as pd
import pyqtgraph as pg
from PyQt5 import QtWidgets

from graphysio import ui
from graphysio.writedata import exporter

Point = namedtuple('Point', ['x', 'y'])
Cardinals = namedtuple('Cardinals', ['A', 'B', 'C'])
Angles = namedtuple('Angles', ['alpha', 'beta', 'gala'])


class LoopWidget(ui.Ui_LoopWidget, QtWidgets.QWidget):
    def __init__(self, u, p, subsetrange, parent):
        super().__init__(parent=parent)
        self.setupUi(self)
        self.properties = {}

        self.subsetrange = subsetrange

        self.exporter = exporter.PuExporter(self, p.name())

        self.btnPrev.clicked.connect(self.prevloop)
        self.btnNext.clicked.connect(self.nextloop)
        self.btnDelete.clicked.connect(self.delloop)

        self.curidx = 0
        self.loops = []
        self.pen = p.opts['pen']
        self.graphicsView.setBackground('w')

        plotitem = self.graphicsView.getPlotItem()
        vb = plotitem.getViewBox()
        vb.setAspectLocked(lock=True, ratio=1)

        self.curveitem = pg.PlotCurveItem()
        self.scatteritem = pg.ScatterPlotItem()
        plotitem.addItem(self.scatteritem)
        plotitem.addItem(self.curveitem)

        self.initloopdata(u, p)

        if self.loops:
            self.lblTot.setText(str(len(self.loops)))
            self.renderloop(0)

    def initloopdata(self, u, p):
        ubegins, udurations = u.getCycleIndices(self.subsetrange)
        pbegins, pdurations = p.getCycleIndices(self.subsetrange)
        durations = map(min, zip(udurations, pdurations))
        for ubegin, pbegin, duration in zip(ubegins, pbegins, durations):
            loopu = u.series.loc[ubegin : ubegin + duration]
            loopp = p.series.loc[pbegin : pbegin + duration]
            self.loops.append(PULoop(loopu, loopp))

    def renderloop(self, idx=None):
        if idx is None:
            idx = self.curidx

        try:
            curloop = self.loops[idx]
        except IndexError:
            return

        self.lblIdx.setText(str(idx + 1))

        delay = int(curloop.offset / 1e6)  # ns to ms
        self.lblDelay.setText(str(delay))

        round1 = partial(round, ndigits=1)
        alpha, beta, gala = map(round1, curloop.angles)
        self.lblAlpha.setText(str(alpha))
        self.lblBeta.setText(str(beta))
        self.lblGala.setText(str(gala))

        card = curloop.cardpoints
        cardx, cardy = zip(*card)

        self.curveitem.setData(curloop.u.values, curloop.p.values, pen=self.pen)
        self.scatteritem.setData(np.array(cardx), np.array(cardy))

    def prevloop(self):
        idx = self.curidx - 1
        if idx >= 0:
            self.curidx = idx
            self.renderloop()

    def nextloop(self):
        idx = self.curidx + 1
        if idx < len(self.loops):
            self.curidx = idx
            self.renderloop()

    def delloop(self):
        try:
            self.loops.pop(self.curidx)
        except IndexError:
            return
        if self.curidx >= len(self.loops):
            # Handle the case where we deleted the last item
            self.curidx = len(self.loops) - 1
        self.lblTot.setText(str(len(self.loops)))
        self.renderloop()

    @property
    def menu(self):
        return {'Export': {'&Loop Data to CSV directory': self.exporter.exportloops}}


class PULoop(object):
    def __init__(self, u, p):
        self.__angles = None
        self.__cardpoints = None

        # Realign pressure and flow
        offset = p.index[0] - u.index[0]
        u.index += offset
        self.offset = abs(offset)

        df = pd.concat([u, p], axis=1)
        self.df = df.interpolate(method='index')

        self.u = self.df[u.name]
        self.p = self.df[p.name]

    @property
    def cardpoints(self):
        if self.__cardpoints is None:
            idxpmax = self.p.idxmax()
            idxvmax = self.u.idxmax()
            A = Point(self.u.iloc[0], self.p.iloc[0])
            B = Point(self.u.loc[idxvmax], self.p.loc[idxvmax])
            C = Point(self.u.loc[idxpmax], self.p.loc[idxpmax])
            self.__cardpoints = Cardinals(A, B, C)
        return self.__cardpoints

    @property
    def angles(self):
        if self.__angles is None:
            card = self.cardpoints
            alpha = calcangle(card.A, card.B)
            beta = calcangle(card.A, card.B, card.C)
            gala = calcangle(card.A, card.C)
            self.__angles = Angles(alpha, beta, gala)
        return self.__angles


def calcangle(looporigin, pointb, pointa=None):
    orig = complex(looporigin.x, looporigin.y)
    cb = complex(pointb.x, pointb.y) - orig
    ca = complex(pointa.x, pointa.y) - orig if pointa else complex(1, 0)
    angca = np.angle(ca, deg=True)
    angcb = np.angle(cb, deg=True)
    return abs(angca - angcb)



from typing import List

import pandas as pd

from graphysio.plotwidgets.curves import CurveItem


def curves_to_parquet(
    curves: List[CurveItem], filepath: str, index_label: str = 'timens'
) -> None:
    sers = [c.series for c in curves]
    data = pd.concat(sers, axis=1).sort_index()
    data.index = data.index.astype('M8[ns]')
    data.to_parquet(filepath, compression='gzip', allow_truncated_timestamps=True)



from .csv import curves_to_csv
from .edf import curves_to_edf
from .matlab import curves_to_matlab
from .parquet import curves_to_parquet

curve_writers = {
    'csv': curves_to_csv,
    'edf': curves_to_edf,
    'parquet': curves_to_parquet,
    'mat': curves_to_matlab,
}



import string
from functools import partial

import pandas as pd
import pyqtgraph as pg
import sympy
from pyqtgraph.Qt import QtCore, QtGui

from graphysio import dialogs, transformations
from graphysio.algorithms import filters
from graphysio.plotwidgets import (
    LoopWidget,
    PlotWidget,
    POISelectorWidget,
    SpectrogramWidget,
)
from graphysio.structures import CycleId, Parameter, PlotData
from graphysio.writedata import exporter


class TSWidget(PlotWidget):
    @staticmethod
    def mouseMoved(self, evt):
        pos = evt[
            0
        ]  ## using signal proxy turns original arguments into a tuple
        if self.sceneBoundingRect().contains(pos):
            mousePoint = self.getViewBox().mapSceneToView(pos)
            self.parent.setcoords.emit(mousePoint.x(), mousePoint.y())

    def __init__(self, plotdata, parent=None, properties={}):
        super().__init__(plotdata.name, parent=parent, properties=properties)
        self.exporter = exporter.TsExporter(self, plotdata.name)
        self.appendData(plotdata)
        mouseMoved = partial(self.mouseMoved, self)
        self.sigproxy = pg.SignalProxy(
            self.scene().sigMouseMoved, rateLimit=60, slot=mouseMoved
        )

    def filterCurve(self, oldcurve, filtername, asnew=False):
        newseries, newsamplerate = filters.filter(
            oldcurve, filtername, dialogs.askUserValue
        )
        if asnew:
            newname = self.validateNewCurveName(newseries.name)
            if newname != newseries.name:
                newseries = newseries.rename(newname)
            newcurve = self.addSeriesAsCurve(series=newseries)
            newcurve.samplerate = newsamplerate
        else:
            newseries = newseries.rename(oldcurve.series.name)
            newseries = newseries.dropna()
            if len(newseries) < 1:
                return
            newseries = newseries.groupby(newseries.index).mean()
            oldcurve.clear()
            oldcurve.series = newseries
            oldcurve.samplerate = newsamplerate
            oldcurve.render()

    def filterFeet(self, curve, filtername, asnew=False):
        feetdict = curve.feetitem.indices
        oldstarts = feetdict['start']
        oldstops = feetdict['stop']
        starts, stops = filters.filterFeet(
            oldstarts, oldstops, filtername, dialogs.askUserValue
        )
        feetdict['start'] = starts
        feetdict['stop'] = stops
        curve.feetitem.render()

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Delete:
            for curve in self.curves.values():
                curve.feetitem.removeSelection()

    # Menu Curves
    def showCurveList(self):
        dlgCurveSelection = dialogs.DlgCurveSelection(
            parent=self.parent,
            visible=self.curves.values(),
            hidden=self.hiddencurves,
        )

        def cb(result):
            currently_visible = set(self.curves.values())
            currently_hidden = self.hiddencurves
            allcurves = currently_visible | currently_hidden
            newly_visible, curveproperties = result
            newly_hidden = allcurves - newly_visible

            for c in curveproperties:
                self.applyCurveProperties(c, curveproperties[c])

            for c in newly_visible:
                self.addCurve(c)

            for c in newly_hidden:
                self.removeCurve(c)

            self.hiddencurves = newly_hidden
            self.rebuildLegend()

        dlgCurveSelection.dlgdata.connect(cb)
        dlgCurveSelection.exec_()

    def launchCycleDetection(self):
        dlgCycles = dialogs.DlgCycleDetection(parent=self.parent)

        def cb(choices):
            for curvename, choice in choices.items():
                curve = self.curves[curvename]
                curve.addFeet(CycleId(choice))

        dlgCycles.dlgdata.connect(cb)
        dlgCycles.exec_()

    def launchFilter(self, filterfeet):
        dlgFilter = dialogs.DlgFilter(parent=self.parent, filterfeet=filterfeet)

        def cb(result):
            createnew, curvechoices = result
            filterfunc = self.filterFeet if filterfeet else self.filterCurve
            for curvename, choice in curvechoices.items():
                if choice == 'None':
                    continue
                curve = self.curves[curvename]
                filterfunc(curve, choice, asnew=createnew)

        dlgFilter.dlgdata.connect(cb)
        dlgFilter.exec_()

    def setDateTime(self):
        if len(self.curves) < 1:
            return
        sortedcurves = sorted(
            self.curves.values(), key=lambda curve: curve.series.index[0]
        )
        fstcurve = sortedcurves[0]
        curtimestamp = fstcurve.series.index[0]
        dlg = dialogs.DlgSetDateTime(prevdatetime=curtimestamp)
        dlg.exec_()
        newtimestamp = dlg.dlgdata
        if not newtimestamp:
            return
        offset = newtimestamp - curtimestamp
        for curve in self.curves.values():
            curve.series.index += offset
            curve.render()

    def launchTransformation(self):
        param = Parameter(
            "Choose Transformation",
            list(transformations.Transformations.keys()),
        )
        qresult = dialogs.askUserValue(param)
        if qresult is None:
            return
        trans = transformations.Transformations[qresult]
        for curve in trans(self):
            self.addCurve(curve)

    # Menu Plot
    def launchPOIWidget(self):
        curvenames = list(self.curves.keys())
        q = Parameter('Select Curve', curvenames)
        curvename = dialogs.askUserValue(q)
        if not curvename:
            return
        curve = self.curves[curvename]
        poiselector = POISelectorWidget(
            curve.series, parent=self.parent, properties=self.properties
        )
        self.parent.addTab(poiselector, curve.name())

    def launchSpectrogram(self):
        curvenames = list(self.curves.keys())
        q = Parameter('Select Curve', curvenames)
        curvename = dialogs.askUserValue(q)
        q = Parameter('Enter time window', 'time')
        window = dialogs.askUserValue(q)
        if not curvename:
            return
        curve = self.curves[curvename]
        spectro = SpectrogramWidget(
            curve.series, curve.samplerate, window, parent=self.parent
        )
        self.parent.addTab(spectro, curve.name())

    def launchCurveAlgebra(self):
        curvecorr = {
            n: c for n, c in zip(string.ascii_lowercase, self.curves.keys())
        }
        dlgCurveAlgebra = dialogs.DlgCurveAlgebra(self, curvecorr)

        def cb(formula):
            expr = sympy.sympify(formula)
            symbols = list(expr.free_symbols)
            schar = map(str, symbols)
            curvenames = [curvecorr[x] for x in schar]
            sers = [self.curves[c].series for c in curvenames]
            if len(sers) < 1:
                # Scalar
                return
            df = pd.concat(sers, axis=1, keys=[s.name for s in sers])
            df = df.dropna(how='all').interpolate()
            args = [df[c].values for c in curvenames]
            l = sympy.lambdify(symbols, expr, 'numpy')
            newvals = l(*args)
            newname = self.validateNewCurveName(formula, True)
            newseries = pd.Series(
                newvals, index=df.index, name=newname, dtype='float64'
            )
            self.addSeriesAsCurve(newseries)

        dlgCurveAlgebra.dlgdata.connect(cb)
        dlgCurveAlgebra.exec_()

    # Menu Selection
    def launchNewPlotFromSelection(self):
        xmin, xmax = self.vbrange
        sers = []
        for c in self.curves.values():
            series = c.series.loc[xmin:xmax]
            sers.append(series)
        df = pd.concat(sers, axis=1, keys=[s.name for s in sers])
        newname = f'{self.name}-sub'
        plotdata = PlotData(data=df, name=newname)
        self.parent.createNewPlotWithData(plotdata)

    def launchAppendToPlotFromSelection(self):
        tabWidget = self.parent.tabWidget
        ntabs = tabWidget.count()
        tabdict = {tabWidget.tabText(idx): idx for idx in range(ntabs)}
        desttabname, ok = QtGui.QInputDialog.getItem(
            self,
            'Select destination',
            'Destination plot',
            list(tabdict.keys()),
            editable=False,
        )
        if not ok:
            return
        xmin, xmax = self.vbrange
        sers = []
        for c in self.curves.values():
            series = c.series.loc[xmin:xmax]
            sers.append(series)
        df = pd.concat(sers, axis=1, keys=[s.name for s in sers])
        newname = f'{self.name}-sub'
        plotdata = PlotData(data=df, name=newname)
        self.parent.appendToPlotWithData(plotdata, destidx=tabdict[desttabname])

    def launchLoop(self):
        dlgSetupPU = dialogs.DlgSetupPULoop(self, parent=self.parent)

        def cb(result):
            uname, pname = result
            u = self.curves[uname]
            p = self.curves[pname]
            loopwidget = LoopWidget(u, p, self.vbrange, parent=self.parent)
            newname = f'{self.name}-{p.name()}-loops'
            self.parent.addTab(loopwidget, newname)

        dlgSetupPU.dlgdata.connect(cb)
        dlgSetupPU.exec_()

    @property
    def menu(self):
        mcurves = {
            'Visible Curves': self.showCurveList,
            'Cycle Detection': self.launchCycleDetection,
            'Filter Curve': partial(self.launchFilter, filterfeet=False),
            'Filter Feet': partial(self.launchFilter, filterfeet=True),
            'Transformation': self.launchTransformation,
            'Set Date / Time': self.setDateTime,
        }
        mselect = {
            'As new plot': self.launchNewPlotFromSelection,
            'Append to other plot': self.launchAppendToPlotFromSelection,
            'Generate PU-Loops': self.launchLoop,
        }
        mplot = {
            'POI Selector': self.launchPOIWidget,
            'Spectrogram': self.launchSpectrogram,
            'Curve Algebra': self.launchCurveAlgebra,
        }
        mexport = {
            'Curves': self.exporter.curves,
            'Time info': self.exporter.periods,
            'Cycle info': self.exporter.cyclepoints,
            'Cycles to directory': self.exporter.cycles,
        }
        return {
            'Curves': mcurves,
            'Plot': mplot,
            'Seletion': mselect,
            'Export': mexport,
        }



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'datetime.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SetDateTime(object):
    def setupUi(self, SetDateTime):
        SetDateTime.setObjectName("SetDateTime")
        SetDateTime.resize(574, 366)
        self.gridLayout = QtWidgets.QGridLayout(SetDateTime)
        self.gridLayout.setObjectName("gridLayout")
        self.widgetCalendar = QtWidgets.QCalendarWidget(SetDateTime)
        self.widgetCalendar.setFirstDayOfWeek(QtCore.Qt.Monday)
        self.widgetCalendar.setGridVisible(True)
        self.widgetCalendar.setObjectName("widgetCalendar")
        self.gridLayout.addWidget(self.widgetCalendar, 0, 2, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 0, 1, 1)
        self.edDate = QtWidgets.QDateEdit(SetDateTime)
        self.edDate.setObjectName("edDate")
        self.gridLayout.addWidget(self.edDate, 1, 2, 1, 1)
        self.edTime = QtWidgets.QTimeEdit(SetDateTime)
        self.edTime.setObjectName("edTime")
        self.gridLayout.addWidget(self.edTime, 2, 2, 1, 1)
        self.label_2 = QtWidgets.QLabel(SetDateTime)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1)
        self.label = QtWidgets.QLabel(SetDateTime)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 1, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.btnOk = QtWidgets.QPushButton(SetDateTime)
        self.btnOk.setObjectName("btnOk")
        self.horizontalLayout.addWidget(self.btnOk)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.gridLayout.addLayout(self.horizontalLayout, 3, 0, 1, 1)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.btnCancel = QtWidgets.QPushButton(SetDateTime)
        self.btnCancel.setObjectName("btnCancel")
        self.horizontalLayout_2.addWidget(self.btnCancel)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem4)
        self.gridLayout.addLayout(self.horizontalLayout_2, 3, 2, 1, 1)
        self.widgetCalendar.raise_()
        self.edTime.raise_()
        self.label_2.raise_()
        self.label.raise_()
        self.edDate.raise_()

        self.retranslateUi(SetDateTime)
        self.widgetCalendar.clicked['QDate'].connect(self.edDate.setDate)
        QtCore.QMetaObject.connectSlotsByName(SetDateTime)

    def retranslateUi(self, SetDateTime):
        _translate = QtCore.QCoreApplication.translate
        SetDateTime.setWindowTitle(_translate("SetDateTime", "Set Date and Time"))
        self.edDate.setDisplayFormat(_translate("SetDateTime", "dd/MM/yyyy"))
        self.edTime.setDisplayFormat(_translate("SetDateTime", "hh:mm:ss.zzz"))
        self.label_2.setText(_translate("SetDateTime", "Time"))
        self.label.setText(_translate("SetDateTime", "Date"))
        self.btnOk.setText(_translate("SetDateTime", "Ok"))
        self.btnCancel.setText(_translate("SetDateTime", "Cancel"))



from datetime import datetime
from typing import List

import numpy as np
import pyedflib
from scipy import interpolate

from graphysio.dialogs import askUserValue
from graphysio.plotwidgets.curves import CurveItem
from graphysio.structures import Parameter


def interp_series(s, samplerate, start, stop):
    f = interpolate.interp1d(
        s.index, s.values, assume_sorted=True, copy=False, fill_value='extrapolate'
    )
    # Recalculate the number of points needed over the duration to account for periods of missing data.
    duration_s = (stop - start) * 1e-9
    npoints = duration_s * samplerate
    newidx = np.linspace(start, stop, num=npoints)
    return f(newidx)


def curves_to_edf(
    curves: List[CurveItem], filepath: str, index_label: str = 'timens'
) -> None:
    headers = []
    signals = []

    beginns = min(c.series.index[0] for c in curves)
    endns = max(c.series.index[-1] for c in curves)
    begindt = datetime.fromtimestamp(beginns * 1e-9)

    # Use global min / max of values since some viewers need this (edfbrowser)
    physmax = max(c.series.max() for c in curves)
    physmin = min(c.series.min() for c in curves)

    # Ask the user for the physical dimension shared by all curves
    dim = askUserValue(Parameter('Enter physical dimension', str))

    for c in curves:
        s = c.series
        header = {
            'label': c.name(),
            'sample_rate': c.samplerate,
            'physical_max': physmax,
            'physical_min': physmin,
            'digital_max': 32767,
            'digital_min': -32768,
            'transducer': '',
            'prefilter': '',
            'dimension': dim,
        }
        headers.append(header)
        resampled = interp_series(s, c.samplerate, beginns, endns)
        signals.append(resampled)

    edf = pyedflib.EdfWriter(
        str(filepath), len(signals), file_type=pyedflib.FILETYPE_EDFPLUS
    )
    edf.setStartdatetime(begindt)
    edf.setSignalHeaders(headers)
    edf.writeSamples(signals)
    edf.close()



import numpy as np
import pandas as pd
import pyedflib

from graphysio.dialogs import DlgListChoice
from graphysio.readdata.baseclass import BaseReader
from graphysio.structures import PlotData


class EdfReader(BaseReader):
    def askUserInput(self):
        filepath = str(self.userdata['filepath'])
        edf = pyedflib.EdfReader(filepath)

        signals = {}
        for i in range(edf.signals_in_file):
            h = edf.getSignalHeader(i)
            signals[h['label']] = i
        edf.close()

        def cb(colnames):
            self.userdata['columns'] = [signals[lbl] for lbl in colnames]

        colnames = list(signals.keys())
        dlgchoice = DlgListChoice(colnames, 'Open EDF', 'Choose curves to load')
        dlgchoice.dlgdata.connect(cb)
        dlgchoice.exec_()

    def __call__(self) -> PlotData:
        filepath = str(self.userdata['filepath'])
        edf = pyedflib.EdfReader(filepath)
        beginns = edf.getStartdatetime().timestamp() * 1e9
        nsamplesPerChannel = edf.getNSamples()

        signals = []
        for i in self.userdata['columns']:
            h = edf.getSignalHeader(i)
            fs = h['sample_rate']
            n = nsamplesPerChannel[i]
            endns = beginns + n * 1e9 / fs
            idx = np.linspace(beginns, endns, num=n, dtype=np.int64)
            s = pd.Series(edf.readSignal(i), index=idx, name=h['label'])
            signals.append(s)
        edf.close()

        if not signals:
            return None

        df = pd.concat(signals, axis=1)
        return PlotData(data=df, filepath=filepath)



import numpy as np
import pandas as pd
import pyarrow.parquet as pa

from graphysio.dialogs import DlgListChoice
from graphysio.readdata.baseclass import BaseReader
from graphysio.structures import PlotData


class ParquetReader(BaseReader):
    def askUserInput(self):
        filepath = self.userdata['filepath']
        s = pa.read_schema(filepath)
        colnames = s.names

        def cb(columns):
            self.userdata['columns'] = columns

        dlgchoice = DlgListChoice(colnames, 'Open Parquet', 'Choose curves to load')
        dlgchoice.dlgdata.connect(cb)
        dlgchoice.exec_()

    def __call__(self) -> PlotData:
        filepath = self.userdata['filepath']
        data = pd.read_parquet(filepath, columns=self.userdata['columns'])

        data = data.dropna(axis='columns', how='all')
        data = data.sort_index()
        data.index = data.index.astype(np.int64)

        return PlotData(data=data, filepath=filepath)



from enum import Enum
from functools import partial

import pandas as pd
import pyqtgraph as pg
from PyQt5 import QtGui, QtWidgets

from graphysio import ui
from graphysio.algorithms.filters import savgol
from graphysio.algorithms.waveform import findPOIGreedy
from graphysio.plotwidgets import PlotWidget
from graphysio.structures import PlotData
from graphysio.writedata import exporter


class FixIndex(Enum):
    disabled = 'Disabled'
    minimum = 'Local Minimum'
    maximum = 'Local Maximum'
    sndderiv = '2nd derivative peak'


class POISelectorWidget(ui.Ui_POISelectorWidget, QtWidgets.QWidget):
    @staticmethod
    def buttonClicked(self, qbutton):
        fixval = FixIndex(qbutton.text())
        self.poiselectorwidget.fixvalue = fixval

    def __init__(self, series, parent, properties={}):
        super().__init__(parent=parent)
        self.setupUi(self)
        self.parent = parent

        self.poiselectorplot = POISelectorPlot(
            series, parent=self, properties=properties
        )
        self.horizontalLayout.addWidget(self.poiselectorplot)

        buttonClicked = partial(self.buttonClicked, self)
        self.buttonGroup.buttonClicked.connect(buttonClicked)

    def loadPOI(self, plotdata):
        columnname, ok = QtGui.QInputDialog.getItem(
            self, 'Select POI series', 'Load POI', plotdata.data.columns, editable=False
        )
        if not ok:
            return
        poiseries = plotdata.data[columnname].dropna().index
        self.poiselectorplot.curve.feetitem.addPointsByLocation(
            self.poiselectorplot.pointkey, poiseries
        )

    def launchNewPlotFromPOIs(self):
        srcseries = self.poiselectorplot.curve.series
        poiidx = self.poiselectorplot.curve.feetitem.indices[
            self.poiselectorplot.pointkey
        ]
        pois = srcseries[poiidx.dropna()].rename('poi')
        sname = f'{srcseries.name}-poi'
        df = pd.DataFrame({sname: pois})
        plotdata = PlotData(data=df, name=sname)
        self.parent.createNewPlotWithData(plotdata)

    @property
    def menu(self):
        return {
            'Plot': {
                'Import POIs': partial(
                    self.parent.launchOpenFile, datahandler=self.loadPOI
                ),
                'POIs to New Plot': self.launchNewPlotFromPOIs,
            },
            'Export': {'POI to CSV': self.poiselectorplot.exporter.poi},
        }


class POISelectorPlot(PlotWidget):
    pointkey = 'point'

    @staticmethod
    def mouseMoved(self, evt):
        pos = evt[0]  ## using signal proxy turns original arguments into a tuple
        if self.sceneBoundingRect().contains(pos):
            mousePoint = self.getViewBox().mapSceneToView(pos)
            self.vLine.setPos(mousePoint.x())

    @staticmethod
    def clicked(self, evt):
        button = evt.button()
        pos = self.vLine.value()
        if button == 1:
            correctedpos = self.fixpos(pos)
            self.curve.feetitem.addPointsByLocation(self.pointkey, [correctedpos])
        elif button == 2:
            self.curve.feetitem.removePointsByLocation(self.pointkey, [pos])

    def __init__(self, series, parent, properties={}):
        super().__init__(name=series.name, parent=parent)
        self.vb.setMouseMode(self.vb.PanMode)
        self.setMenuEnabled(False)
        self.properties = properties

        self.__sndderiv = None
        self.fixvalue = FixIndex.disabled

        self.curve = self.addSeriesAsCurve(series)
        self.exporter = exporter.POIExporter(self, self.name)

        pen = pg.mkPen('k', width=2)
        self.vLine = pg.InfiniteLine(angle=90, movable=False, pen=pen)
        self.addItem(self.vLine, ignoreBounds=True)

        mouseMoved = partial(self.mouseMoved, self)
        self.sigproxy = pg.SignalProxy(
            self.scene().sigMouseMoved, rateLimit=60, slot=mouseMoved
        )
        clicked = partial(self.clicked, self)
        self.scene().sigMouseClicked.connect(clicked)

    def fixpos(self, pos):
        # Need to go through get_loc, otherwise strange things happen
        correctedpos = pos
        xmin, xmax = self.vbrange
        if self.fixvalue is FixIndex.minimum:
            s = self.curve.series.loc[xmin:xmax]
            posprop = findPOIGreedy(s, pos, 'min')
            fixedposloc = s.index.get_loc(posprop, method='nearest')
        elif self.fixvalue is FixIndex.maximum:
            s = self.curve.series.loc[xmin:xmax]
            posprop = findPOIGreedy(s, pos, 'max')
            fixedposloc = s.index.get_loc(posprop, method='nearest')
        elif self.fixvalue is FixIndex.sndderiv:
            s = self.sndderiv.loc[xmin:xmax]
            posprop = findPOIGreedy(s, pos, 'max')
            fixedposloc = s.index.get_loc(posprop, method='nearest')
        else:
            s = self.curve.series.loc[xmin:xmax]
            fixedposloc = s.index.get_loc(pos, method='nearest')
        correctedpos = s.index[fixedposloc]
        return correctedpos

    @property
    def sndderiv(self):
        if self.__sndderiv is None:
            sndderiv = self.curve.series.diff().diff().iloc[1:]
            sndderiv, _ = savgol(sndderiv, self.curve.samplerate, (0.16, 2))
            self.__sndderiv = sndderiv.dropna()
        return self.__sndderiv



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mainwindow.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800, 630)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setDocumentMode(True)
        self.tabWidget.setTabsClosable(True)
        self.tabWidget.setObjectName("tabWidget")
        self.verticalLayout.addWidget(self.tabWidget)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lblStatus = QtWidgets.QLabel(self.centralwidget)
        self.lblStatus.setMinimumSize(QtCore.QSize(150, 0))
        self.lblStatus.setText("")
        self.lblStatus.setObjectName("lblStatus")
        self.horizontalLayout.addWidget(self.lblStatus)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.lblCoords = QtWidgets.QLabel(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblCoords.sizePolicy().hasHeightForWidth())
        self.lblCoords.setSizePolicy(sizePolicy)
        self.lblCoords.setMinimumSize(QtCore.QSize(150, 0))
        self.lblCoords.setText("")
        self.lblCoords.setObjectName("lblCoords")
        self.horizontalLayout.addWidget(self.lblCoords)
        self.verticalLayout.addLayout(self.horizontalLayout)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menuBar = QtWidgets.QMenuBar(MainWindow)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 800, 22))
        self.menuBar.setObjectName("menuBar")
        self.menuFile = QtWidgets.QMenu(self.menuBar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menuBar)
        self.actNewplot = QtWidgets.QAction(MainWindow)
        self.actNewplot.setObjectName("actNewplot")
        self.actQuit = QtWidgets.QAction(MainWindow)
        self.actQuit.setObjectName("actQuit")
        self.menuBar.addAction(self.menuFile.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(-1)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "GraPhysio"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.actNewplot.setText(_translate("MainWindow", "New plot"))
        self.actQuit.setText(_translate("MainWindow", "Quit"))



import sys

from pyqtgraph.Qt import QtGui

from graphysio.mainui import MainUi


def main():
    app = QtGui.QApplication(sys.argv)

    winmain = MainUi()
    winmain.show()

    sys.exit(app.exec_())



"""Graphical time series visualizer and analyzer."""

__version__ = '2021.07.14'

__all__ = [
    'algorithms',
    'dialogs',
    'exporter',
    'legend',
    'mainui',
    'puplot',
    'tsplot',
    'utils',
    'types',
    'ui',
    'transformations',
]



import numpy as np
import pandas as pd
import pyqtgraph as pg
from pyqtgraph.Qt import QtCore, QtWidgets

from graphysio import dialogs
from graphysio.structures import Parameter, PlotData


class SpectrogramWidget(QtWidgets.QWidget):
    def __init__(self, series, Fs, s_chunklen, parent=None):
        super().__init__(parent=parent)
        self.spectro = SpectrogramPlotWidget(series, Fs, s_chunklen, parent)
        self.loslider = QtWidgets.QSlider()
        self.hislider = QtWidgets.QSlider()
        layout = QtWidgets.QHBoxLayout()

        lolayout = QtWidgets.QVBoxLayout()
        lothres = QtWidgets.QLabel('Low')
        lolayout.addWidget(lothres)
        lolayout.addWidget(self.loslider)

        hilayout = QtWidgets.QVBoxLayout()
        hithres = QtWidgets.QLabel('High')
        hilayout.addWidget(hithres)
        hilayout.addWidget(self.hislider)

        layout.addWidget(self.spectro)
        layout.addLayout(lolayout)
        layout.addLayout(hilayout)
        self.setLayout(layout)

    @property
    def menu(self):
        mplot = {'Extract SEF': self.spectro.launchSEFExtract}
        return {'Plot': mplot}


class SpectroTimeAxisItem(pg.AxisItem):
    def __init__(self, initvalue, samplerate, chunksize, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.initvalue = initvalue / 1e6  # ns to ms
        self.samplerate = samplerate
        self.chunksize = chunksize

    def tickStrings(self, values, scale, spacing):
        ret = []
        value_to_time = self.chunksize / self.samplerate + self.initvalue
        for value in values:
            value = 1e3 * value  # s to ms
            value = value * value_to_time
            date = QtCore.QDateTime.fromMSecsSinceEpoch(value)
            date = date.toTimeSpec(QtCore.Qt.UTC)
            datestr = date.toString("dd/MM/yyyy\nhh:mm:ss.zzz")
            ret.append(datestr)
        return ret


# To set:
# levels
# color gradient
class SpectrogramPlotWidget(pg.PlotWidget):
    def __init__(self, series, Fs, s_chunklen, parent=None):
        # s_chunklen in seconds
        self.name = series.name
        self.parent = parent
        self.origidx = series.index.values

        self.fs = Fs
        self.data = series.values
        self.chunksize = int(s_chunklen * Fs)
        self.win = np.hanning(self.chunksize)

        axisItem = SpectroTimeAxisItem(
            initvalue=self.origidx[0],
            samplerate=self.fs,
            chunksize=self.chunksize,
            orientation='bottom',
        )
        axisItems = {'bottom': axisItem}
        super().__init__(parent=self.parent, axisItems=axisItems)

        self.img = pg.ImageItem()
        self.addItem(self.img)
        self.setLabel('left', 'Frequency', units='Hz')

        # bipolar colormap
        pos = np.array([0.0, 0.25, 0.5, 0.75, 1.0])
        color = np.array(
            [
                [0, 0, 0, 255],
                [0, 0, 255, 255],
                [0, 255, 255, 255],
                [255, 255, 0, 255],
                [255, 0, 0, 255],
            ],
            dtype=np.ubyte,
        )
        cmap = pg.ColorMap(pos, color)
        lut = cmap.getLookupTable(0.0, 1.0, 256)
        self.img.setLookupTable(lut)

        self.calculate_psd()
        self.render()

    def idx_to_hz(self, idxval):
        conv = self.fs / self.chunksize
        return conv * idxval

    def calcsef(self, perc):
        buf = np.zeros(self.psd.shape[0])
        psdcs = np.cumsum(self.psd, axis=1)
        for i, columncs in enumerate(psdcs):
            thres = perc * columncs[-1] / 100
            sefidx = np.where(columncs > thres)
            try:
                buf[i] = self.idx_to_hz(sefidx[0][0])
            except IndexError:
                pass
        return buf

    def calculate_psd(self):
        nsplit = int(len(self.data) / self.chunksize)
        chunks = self.data[0 : nsplit * self.chunksize]
        chunks = np.split(chunks, nsplit)
        chunks = np.vstack(chunks)
        spec = np.fft.rfft(chunks * self.win) / self.chunksize
        self.psd = np.real(spec) ** 2

    def genIndex(self):
        nwindows = self.psd.shape[0]
        winsize = self.chunksize
        windows = np.lib.stride_tricks.as_strided(
            self.origidx, shape=(nwindows, winsize)
        )
        result = np.apply_along_axis(np.mean, 1, windows)
        return result.astype(int)

    def launchSEFExtract(self):
        q = Parameter('SEF percentage', int)
        sefperc = dialogs.askUserValue(q)
        if not sefperc:
            return
        curvename = f'{self.name}-sef{sefperc}'
        sef = self.calcsef(sefperc)
        sefseries = pd.Series(sef, index=self.genIndex(), name=curvename)
        data = {curvename: sefseries}
        plotdata = PlotData(data, name=curvename)
        self.parent.createNewPlotWithData(plotdata)

    def render(self):
        # TODO make lo / hi adjustable
        psdnona = self.psd[~np.isnan(self.psd)]
        hi = np.percentile(psdnona, 95)
        lo = np.percentile(psdnona, 5)
        # print(f'PSD Lo: {lo}, Hi: {hi}')
        self.img.scale(1, self.fs / self.chunksize)
        self.img.setLevels([lo, hi])
        self.img.setImage(self.psd, autoLevels=False)



import csv
import os
from itertools import zip_longest

import pandas as pd

from graphysio import writedata
from graphysio.dialogs import DlgPeriodExport, askDirPath, askSaveFilePath
from graphysio.utils import sanitize_filename

file_filters = ';;'.join(
    [f'{ext.upper()} files (*.{ext})' for ext in writedata.curve_writers]
)


class TsExporter:
    periodfields = ['patient', 'begin', 'end', 'periodid', 'comment']

    def __init__(self, parent, name):
        self.parent = parent
        self.name = name
        try:
            self.outdir = parent.properties['dircache']
        except KeyError:
            self.outdir = os.path.expanduser('~')

    def curves(self) -> None:
        filepath, ext = askSaveFilePath(
            'Export to', f'{self.name}.csv', self.outdir, filter=file_filters
        )
        if filepath is None:
            return
        self.outdir = os.path.dirname(filepath)
        curves = list(self.parent.curves.values())
        export_func = writedata.curve_writers[ext]
        export_func(curves, filepath)

    def periods(self):
        xmin, xmax = self.parent.vbrange
        dlg = DlgPeriodExport(
            begin=xmin, end=xmax, patient=self.name, directory=self.outdir
        )

        def cb(result):
            patient, comment, periodname, filepath = result
            self.name = patient
            self.outdir = os.path.dirname(filepath)

            with open(filepath, 'a', newline='') as csvfile:
                writer = csv.DictWriter(
                    csvfile, fieldnames=self.periodfields, quoting=csv.QUOTE_MINIMAL
                )
                if not os.path.exists(filepath):
                    writer.writeheader()
                writer.writerow(
                    {
                        'patient': patient,
                        'begin': xmin,
                        'end': xmax,
                        'periodid': periodname,
                        'comment': comment,
                    }
                )

        dlg.dlgdata.connect(cb)
        dlg.exec_()

    def cycles(self) -> None:
        outdir = askDirPath("Export to", self.outdir)
        if outdir is None:
            # Cancel pressed
            return

        # Some non trivial manipulations to get the cycles from all
        # curves, then reorganize to group by the n-th cycle from each
        # curve and put those cycles into a dataframe for export.
        def getCurveCycles(curve):
            cycleIdx = curve.getCycleIndices()
            return (curve.series.loc[b : b + d] for b, d in zip(*cycleIdx))

        curves = self.parent.curves.values()
        allByCurve = (getCurveCycles(curve) for curve in curves)
        allByCycle = zip_longest(*allByCurve)

        for n, cycle in enumerate(allByCycle):
            idxstart = None
            for s in cycle:
                # Make all series start at the same point
                if s is None or len(s) < 1:
                    continue
                if idxstart is None:
                    idxstart = s.index[0]
                idxdelta = s.index[0] - idxstart
                s.index -= idxdelta
            df = pd.concat(cycle, axis=1)
            df['datetime'] = pd.to_datetime(df.index, unit='ns')

            filename = sanitize_filename(f'{self.name}-{n+1}.csv')
            filepath = os.path.join(self.outdir, filename)
            df.to_csv(
                filepath, date_format="%Y-%m-%d %H:%M:%S.%f", index_label='timens'
            )

    def cyclepoints(self) -> None:
        filepath, _ = askSaveFilePath('Export to', f'{self.name}-feet.csv', self.outdir)
        if filepath is None:
            # Cancel pressed
            return
        self.outdir = os.path.dirname(filepath)

        feetseries = []
        for c in self.parent.curves.values():
            for k, v in c.feetitem.indices.items():
                feetseries.append(pd.Series(v, name=f'{c.name()}-{k}'))
        df = pd.concat(feetseries, axis=1)
        df.to_csv(filepath, index_label='idx')


class PuExporter:
    def __init__(self, parent, name):
        self.parent = parent
        self.name = name
        try:
            self.outdir = parent.properties['dircache']
        except KeyError:
            self.outdir = os.path.expanduser('~')

    def exportloops(self):
        outdir = askDirPath('Export to', self.outdir)
        if outdir is None:
            # Cancel pressed
            return
        self.outdir = outdir
        self.writetable()
        self.writeloops()

    def writetable(self):
        data = []
        for loop in self.parent.loops:
            alpha, beta, gala = loop.angles
            delay = loop.offset / 1e6
            tmpdict = {'alpha': alpha, 'beta': beta, 'gala': gala, 'delay': delay}
            data.append(tmpdict)
        df = pd.DataFrame(data)
        df.index += 1
        filename = sanitize_filename(f'{self.name}-loopdata.csv')
        filepath = os.path.join(self.outdir, filename)
        df.to_csv(filepath, index_label='idx')

    def writeloops(self):
        for n, loop in enumerate(self.parent.loops):
            df = loop.df
            filename = sanitize_filename(f'{self.name}-{n+1}.csv')
            filepath = os.path.join(self.outdir, filename)
            df['datetime'] = pd.to_datetime(df.index, unit='ns')
            df.to_csv(
                filepath, date_format="%Y-%m-%d %H:%M:%S.%f", index_label='timens'
            )


class POIExporter:
    def __init__(self, parent, name):
        self.parent = parent
        self.name = name
        try:
            self.outdir = parent.properties['dircache']
        except KeyError:
            self.outdir = os.path.expanduser('~')

    def poi(self):
        filepath, _ = askSaveFilePath('Export to', f'{self.name}-poi.csv', self.outdir)
        if filepath is None:
            # Cancel pressed
            return
        self.outdir = os.path.dirname(filepath)

        srcseries = self.parent.curve.series
        poiidx = self.parent.curve.feetitem.indices[self.parent.pointkey]
        pois = srcseries[poiidx.dropna()].rename('poi')
        sers = [srcseries, pois]
        df = pd.concat(sers, axis=1, keys=[s.name for s in sers])
        df['datetime'] = pd.to_datetime(df.index, unit='ns')
        df.to_csv(filepath, date_format="%Y-%m-%d %H:%M:%S.%f", index_label='timens')



from typing import Optional

import pyqtgraph as pg
from pyqtgraph.Qt import QtCore, QtGui

from graphysio.legend import LegendItem
from graphysio.plotwidgets import curves
from graphysio.utils import Colors


class TimeAxisItem(pg.AxisItem):
    def tickStrings(self, values, scale, spacing):
        ret = []
        for i, value in enumerate(values):
            value = value / 1e6  # convert from ns to ms
            date = QtCore.QDateTime.fromMSecsSinceEpoch(value)
            date = date.toTimeSpec(QtCore.Qt.UTC)
            datestr = date.toString("dd/MM/yyyy\nhh:mm:ss.zzz")
            ret.append(datestr)
        return ret


class PlotWidget(pg.PlotWidget):
    def __init__(self, name, parent=None, properties={}):
        self.parent = parent
        self.name = name
        self.colors = Colors()
        self.hiddencurves = set()
        self.properties = properties

        axisItems = {'bottom': TimeAxisItem(orientation='bottom')}
        super().__init__(parent=parent, axisItems=axisItems, background='w')

        self.legend = LegendItem()
        self.legend.setParentItem(self.getPlotItem())

        self.vb = self.getViewBox()
        self.vb.setMouseMode(self.vb.RectMode)

        self.setCursor(QtCore.Qt.CrossCursor)

    def appendData(self, newplotdata, dorealign=False):
        for seriesname in newplotdata.data:
            self.addSeriesAsCurve(newplotdata.data[seriesname], dorealign=dorealign)

    @property
    def curves(self):
        return {
            item.name(): item
            for item in self.listDataItems()
            if isinstance(item, curves.CurveItem)
        }

    def getPen(self):
        return next(self.colors)

    def addSeriesAsCurve(self, series, pen=None, dorealign=False, withfeet=True):
        if len(series) < 1:
            return
        if dorealign and self.curves:
            # Timeshift new curves to make the beginnings coincide
            begins = [curve.series.index[0] for curve in self.curves.values()]
            offset = min(begins) - series.index[0]
            series.index += offset
        try:
            # Append to existing curve?
            curve = self.curves[series.name]
            curve.extend(series)
        except KeyError:
            # New curve
            if pen is None:
                pen = self.getPen()
            Curve = curves.CurveItemWithPOI if withfeet else curves.CurveItem
            curve = Curve(series=series, parent=self, pen=pen)
            self.addCurve(curve)
        return curve

    def addCurve(self, curve, pen=None):
        if curve.name() in self.curves:
            return
        if pen is not None:
            curve.setPen(pen)
        self.addItem(curve)
        self.rebuildLegend()
        curve.visible.emit()

    def removeCurve(self, curve):
        if curve.name() not in self.curves:
            return
        self.removeItem(curve)
        self.rebuildLegend()
        curve.invisible.emit()

    def rebuildLegend(self):
        self.legend.clear()
        for name, curve in self.curves.items():
            if isinstance(curve, pg.PlotDataItem):
                self.legend.addItem(curve, curve.name())

    def validateNewCurveName(
        self, proposedname: str, alwaysShow: bool = False
    ) -> Optional[str]:
        if proposedname not in self.curves and not alwaysShow:
            return proposedname
        newname, okPressed = QtGui.QInputDialog.getText(
            self,
            "Series name",
            "Series with identical names will be merged.",
            text=proposedname,
        )
        if not okPressed:
            return None
        return newname

    def applyCurveProperties(self, curve, properties: dict) -> None:
        # Setting Symbol and Connect does not work. TODO dig into pyqtgraph
        # curve.setData(symbol=properties['symbol'])
        # curve.setData(connect=properties['connect'], symbol=properties['symbol'])
        curve.setPen(properties['color'], width=properties['width'])
        curve.rename(properties['name'])

    @property
    def vbrange(self):
        vbrange = self.vb.viewRange()
        xmin, xmax = map(int, vbrange[0])
        return (xmin, xmax)

    @property
    def menu(self):
        return {}



from graphysio.dialogs import askOpenFilePath

from .csv import CsvReader
from .edf import EdfReader
from .parquet import ParquetReader

file_readers = {'csv': CsvReader, 'parquet': ParquetReader, 'edf': EdfReader}


class FileReader:
    def __init__(self):
        super().__init__()
        self.reader = None

        filters = ';;'.join(
            [f'{ext.upper()} files (*.{ext})(*.{ext})' for ext in file_readers]
        )
        supported = ' '.join(f'*.{ext}' for ext in file_readers)
        self.file_filters = f'All supported ({supported});;{filters}'

    def askFile(self, folder='') -> 'pathlib.PurePath':
        filepath, ext = askOpenFilePath(
            'Open File', folder=folder, filter=self.file_filters
        )
        if not filepath:
            return folder
        self.reader = file_readers[ext]()
        self.reader.set_data({'filepath': filepath})
        self.reader.askUserInput()
        # Return the parent folder for caching
        return filepath.parent

    # Meant to be executed in seperate thread
    def get_plotdata(self) -> 'PlotData':
        if self.reader:
            return self.reader()
        else:
            return None



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'csvrequest.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_NewPlot(object):
    def setupUi(self, NewPlot):
        NewPlot.setObjectName("NewPlot")
        NewPlot.resize(942, 792)
        NewPlot.setSizeGripEnabled(False)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(NewPlot)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(NewPlot)
        self.label.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.label.setFrameShadow(QtWidgets.QFrame.Plain)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.txtSep = QtWidgets.QComboBox(NewPlot)
        self.txtSep.setEditable(True)
        self.txtSep.setObjectName("txtSep")
        self.txtSep.addItem("")
        self.txtSep.addItem("")
        self.txtSep.addItem("")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.txtSep)
        self.label_2 = QtWidgets.QLabel(NewPlot)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.txtDecimal = QtWidgets.QComboBox(NewPlot)
        self.txtDecimal.setEditable(True)
        self.txtDecimal.setObjectName("txtDecimal")
        self.txtDecimal.addItem("")
        self.txtDecimal.addItem("")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.txtDecimal)
        self.label_3 = QtWidgets.QLabel(NewPlot)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.spnLinedrop = QtWidgets.QSpinBox(NewPlot)
        self.spnLinedrop.setObjectName("spnLinedrop")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.spnLinedrop)
        self.label_4 = QtWidgets.QLabel(NewPlot)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.txtEncoding = QtWidgets.QComboBox(NewPlot)
        self.txtEncoding.setEditable(True)
        self.txtEncoding.setObjectName("txtEncoding")
        self.txtEncoding.addItem("")
        self.txtEncoding.addItem("")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.txtEncoding)
        self.label_5 = QtWidgets.QLabel(NewPlot)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.label_5)
        self.txtTimezone = QtWidgets.QComboBox(NewPlot)
        self.txtTimezone.setEditable(True)
        self.txtTimezone.setObjectName("txtTimezone")
        self.txtTimezone.addItem("")
        self.txtTimezone.addItem("")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.txtTimezone)
        self.verticalLayout_3.addLayout(self.formLayout)
        self.btnLoad = QtWidgets.QPushButton(NewPlot)
        self.btnLoad.setObjectName("btnLoad")
        self.verticalLayout_3.addWidget(self.btnLoad)
        self.line = QtWidgets.QFrame(NewPlot)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_3.addWidget(self.line)
        self.horizontalLayout_4.addLayout(self.verticalLayout_3)
        self.groupBox_2 = QtWidgets.QGroupBox(NewPlot)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_2.sizePolicy().hasHeightForWidth())
        self.groupBox_2.setSizePolicy(sizePolicy)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.lstVAll = QtWidgets.QTableView(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lstVAll.sizePolicy().hasHeightForWidth())
        self.lstVAll.setSizePolicy(sizePolicy)
        self.lstVAll.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.lstVAll.setObjectName("lstVAll")
        self.verticalLayout.addWidget(self.lstVAll)
        self.verticalLayout_11 = QtWidgets.QVBoxLayout()
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btnToX = QtWidgets.QPushButton(self.groupBox_2)
        self.btnToX.setObjectName("btnToX")
        self.horizontalLayout.addWidget(self.btnToX)
        self.btnToY = QtWidgets.QPushButton(self.groupBox_2)
        self.btnToY.setObjectName("btnToY")
        self.horizontalLayout.addWidget(self.btnToY)
        self.verticalLayout_11.addLayout(self.horizontalLayout)
        self.btnToCluster = QtWidgets.QPushButton(self.groupBox_2)
        self.btnToCluster.setObjectName("btnToCluster")
        self.verticalLayout_11.addWidget(self.btnToCluster)
        self.verticalLayout.addLayout(self.verticalLayout_11)
        self.horizontalLayout_4.addWidget(self.groupBox_2)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.groupBox_6 = QtWidgets.QGroupBox(NewPlot)
        self.groupBox_6.setObjectName("groupBox_6")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.groupBox_6)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_7 = QtWidgets.QLabel(self.groupBox_6)
        self.label_7.setObjectName("label_7")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_7)
        self.txtDateTime = QtWidgets.QComboBox(self.groupBox_6)
        self.txtDateTime.setEditable(True)
        self.txtDateTime.setObjectName("txtDateTime")
        self.txtDateTime.addItem("")
        self.txtDateTime.addItem("")
        self.txtDateTime.addItem("")
        self.txtDateTime.addItem("")
        self.txtDateTime.addItem("")
        self.txtDateTime.addItem("")
        self.txtDateTime.addItem("")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.txtDateTime)
        self.lstVX = QtWidgets.QComboBox(self.groupBox_6)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lstVX.sizePolicy().hasHeightForWidth())
        self.lstVX.setSizePolicy(sizePolicy)
        self.lstVX.setMaxCount(4)
        self.lstVX.setObjectName("lstVX")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.lstVX)
        self.btnRemoveX = QtWidgets.QPushButton(self.groupBox_6)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnRemoveX.sizePolicy().hasHeightForWidth())
        self.btnRemoveX.setSizePolicy(sizePolicy)
        self.btnRemoveX.setObjectName("btnRemoveX")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.btnRemoveX)
        self.verticalLayout_2.addLayout(self.formLayout_2)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.chkGenX = QtWidgets.QCheckBox(self.groupBox_6)
        self.chkGenX.setChecked(True)
        self.chkGenX.setObjectName("chkGenX")
        self.horizontalLayout_2.addWidget(self.chkGenX)
        self.spnFs = QtWidgets.QSpinBox(self.groupBox_6)
        self.spnFs.setMinimum(1)
        self.spnFs.setMaximum(1000000)
        self.spnFs.setProperty("value", 180)
        self.spnFs.setObjectName("spnFs")
        self.horizontalLayout_2.addWidget(self.spnFs)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.verticalLayout_8.addLayout(self.verticalLayout_2)
        self.verticalLayout_5.addWidget(self.groupBox_6)
        self.groupBox_5 = QtWidgets.QGroupBox(NewPlot)
        self.groupBox_5.setObjectName("groupBox_5")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.groupBox_5)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.lstVY = QtWidgets.QListView(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lstVY.sizePolicy().hasHeightForWidth())
        self.lstVY.setSizePolicy(sizePolicy)
        self.lstVY.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.lstVY.setObjectName("lstVY")
        self.verticalLayout_6.addWidget(self.lstVY)
        self.btnRemoveY = QtWidgets.QPushButton(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnRemoveY.sizePolicy().hasHeightForWidth())
        self.btnRemoveY.setSizePolicy(sizePolicy)
        self.btnRemoveY.setObjectName("btnRemoveY")
        self.verticalLayout_6.addWidget(self.btnRemoveY)
        self.verticalLayout_7.addLayout(self.verticalLayout_6)
        self.verticalLayout_5.addWidget(self.groupBox_5)
        self.groupBox = QtWidgets.QGroupBox(NewPlot)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.lstVCluster = QtWidgets.QComboBox(self.groupBox)
        self.lstVCluster.setEditable(False)
        self.lstVCluster.setObjectName("lstVCluster")
        self.verticalLayout_9.addWidget(self.lstVCluster)
        self.btnRemoveCluster = QtWidgets.QPushButton(self.groupBox)
        self.btnRemoveCluster.setObjectName("btnRemoveCluster")
        self.verticalLayout_9.addWidget(self.btnRemoveCluster)
        self.verticalLayout_10.addLayout(self.verticalLayout_9)
        self.verticalLayout_5.addWidget(self.groupBox)
        self.horizontalLayout_4.addLayout(self.verticalLayout_5)
        self.verticalLayout_4.addLayout(self.horizontalLayout_4)
        self.line_2 = QtWidgets.QFrame(NewPlot)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_4.addWidget(self.line_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.btnOk = QtWidgets.QPushButton(NewPlot)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnOk.sizePolicy().hasHeightForWidth())
        self.btnOk.setSizePolicy(sizePolicy)
        self.btnOk.setObjectName("btnOk")
        self.horizontalLayout_3.addWidget(self.btnOk)
        self.btnCancel = QtWidgets.QPushButton(NewPlot)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnCancel.sizePolicy().hasHeightForWidth())
        self.btnCancel.setSizePolicy(sizePolicy)
        self.btnCancel.setObjectName("btnCancel")
        self.horizontalLayout_3.addWidget(self.btnCancel)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)

        self.retranslateUi(NewPlot)
        self.chkGenX.clicked['bool'].connect(self.lstVX.setDisabled)
        QtCore.QMetaObject.connectSlotsByName(NewPlot)

    def retranslateUi(self, NewPlot):
        _translate = QtCore.QCoreApplication.translate
        NewPlot.setWindowTitle(_translate("NewPlot", "New Plot"))
        self.label.setText(_translate("NewPlot", "Field Seperator"))
        self.txtSep.setItemText(0, _translate("NewPlot", ";"))
        self.txtSep.setItemText(1, _translate("NewPlot", ","))
        self.txtSep.setItemText(2, _translate("NewPlot", "<tab>"))
        self.label_2.setText(_translate("NewPlot", "Decimal Seperator"))
        self.txtDecimal.setItemText(0, _translate("NewPlot", ","))
        self.txtDecimal.setItemText(1, _translate("NewPlot", "."))
        self.label_3.setText(_translate("NewPlot", "Drop n first lines"))
        self.label_4.setText(_translate("NewPlot", "Encoding"))
        self.txtEncoding.setItemText(0, _translate("NewPlot", "latin1"))
        self.txtEncoding.setItemText(1, _translate("NewPlot", "utf8"))
        self.label_5.setText(_translate("NewPlot", "Timezone"))
        self.txtTimezone.setItemText(0, _translate("NewPlot", "UTC"))
        self.txtTimezone.setItemText(1, _translate("NewPlot", "CET"))
        self.btnLoad.setText(_translate("NewPlot", "Load Fields"))
        self.groupBox_2.setTitle(_translate("NewPlot", "Loaded Fields"))
        self.btnToX.setText(_translate("NewPlot", "Move to X"))
        self.btnToY.setText(_translate("NewPlot", "Move to Y"))
        self.btnToCluster.setText(_translate("NewPlot", "Use as Cluster Id"))
        self.groupBox_6.setTitle(_translate("NewPlot", "X axis (Time)"))
        self.label_7.setText(_translate("NewPlot", "Date/Time Format"))
        self.txtDateTime.setItemText(0, _translate("NewPlot", "%Y-%m-%d %H:%M:%S,%f"))
        self.txtDateTime.setItemText(1, _translate("NewPlot", "%Y-%m-%d %H:%M:%S.%f"))
        self.txtDateTime.setItemText(2, _translate("NewPlot", "%H:%M"))
        self.txtDateTime.setItemText(3, _translate("NewPlot", "<seconds>"))
        self.txtDateTime.setItemText(4, _translate("NewPlot", "<milliseconds>"))
        self.txtDateTime.setItemText(5, _translate("NewPlot", "<nanoseconds>"))
        self.txtDateTime.setItemText(6, _translate("NewPlot", "<infer>"))
        self.btnRemoveX.setText(_translate("NewPlot", "Remove"))
        self.chkGenX.setText(_translate("NewPlot", "Manual sample rate (Hz):"))
        self.groupBox_5.setTitle(_translate("NewPlot", "Y axis"))
        self.btnRemoveY.setText(_translate("NewPlot", "Remove"))
        self.groupBox.setTitle(_translate("NewPlot", "Cluster Id"))
        self.btnRemoveCluster.setText(_translate("NewPlot", "Remove"))
        self.btnOk.setText(_translate("NewPlot", "OK"))
        self.btnCancel.setText(_translate("NewPlot", "Cancel"))



import os
import pathlib
from datetime import datetime
from functools import partial
from typing import Optional

from pint import UnitRegistry
from pint.errors import DimensionalityError, UndefinedUnitError
from pyqtgraph.Qt import QtCore, QtGui, QtWidgets

from graphysio import ui
from graphysio.algorithms import filters
from graphysio.structures import CycleId
from graphysio.utils import sanitize_filepath

ureg = UnitRegistry()


class DlgCycleDetection(ui.Ui_CycleDetection, QtWidgets.QDialog):
    dlgdata = QtCore.pyqtSignal(object)

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)

        self.okButton.clicked.connect(self.accept)
        self.cancelButton.clicked.connect(self.reject)

        self.choices = {}

        plotframe = self.parent().tabWidget.currentWidget()
        if plotframe is None:
            return

        for n, curvename in enumerate(plotframe.curves.keys()):
            combo = QtGui.QComboBox()
            combo.addItems([ft.value for ft in CycleId])
            curveitem = QtGui.QTableWidgetItem(curvename)

            self.table.insertRow(n)
            self.table.setItem(n, 0, curveitem)
            self.table.setCellWidget(n, 1, combo)
            self.choices[curvename] = combo

        self.table.horizontalHeader().setResizeMode(QtGui.QHeaderView.ResizeToContents)

    def accept(self):
        result = {curve: combo.currentText() for (curve, combo) in self.choices.items()}
        self.dlgdata.emit(result)
        super().accept()


class DlgFilter(ui.Ui_Filter, QtWidgets.QDialog):
    dlgdata = QtCore.pyqtSignal(object)

    def __init__(self, parent=None, filterfeet=False):
        super().__init__(parent=parent)
        self.setupUi(self)

        self.okButton.clicked.connect(self.accept)
        self.cancelButton.clicked.connect(self.reject)

        self.choices = {}

        plotframe = self.parent().tabWidget.currentWidget()
        if not plotframe:
            return

        def fillTable(items, itemtype, filters):
            rowoffset = self.table.rowCount()
            for n, itemname in enumerate(items):
                combo = QtGui.QComboBox()

                combo.addItems(['None'])
                filternames = list(filters.keys())
                combo.addItems(filternames)

                curveitem = QtGui.QTableWidgetItem(itemname)

                idx = rowoffset + n
                self.table.insertRow(idx)
                self.table.setItem(idx, 0, curveitem)
                self.table.setCellWidget(idx, 1, combo)
                self.choices[itemname] = (combo, itemtype)

        curves = list(plotframe.curves.keys())
        if filterfeet:
            self.chkNewcurve.hide()
            fillTable(curves, 'curve', filters.FeetFilters)
        else:
            fillTable(curves, 'curve', filters.Filters)

        self.table.horizontalHeader().setResizeMode(QtGui.QHeaderView.ResizeToContents)

    def accept(self):
        curvefilters = {}
        for itemname, value in self.choices.items():
            combo, itemtype = value
            curvefilters[itemname] = combo.currentText()
        createnew = self.chkNewcurve.checkState() > QtCore.Qt.Unchecked
        result = (createnew, curvefilters)
        self.dlgdata.emit(result)
        super().accept()


class DlgSetupPULoop(ui.Ui_SetupPULoop, QtWidgets.QDialog):
    dlgdata = QtCore.pyqtSignal(object)

    def __init__(self, sourcewidget, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)

        self.okButton.clicked.connect(self.accept)
        self.cancelButton.clicked.connect(self.reject)

        try:
            curvenames = list(sourcewidget.curves.keys())
        except AttributeError:
            return

        self.comboU.addItems(curvenames)
        self.comboP.addItems(curvenames)

    def accept(self):
        uname = self.comboU.currentText()
        pname = self.comboP.currentText()
        result = (uname, pname)
        self.dlgdata.emit(result)
        super().accept()


class DlgPeriodExport(ui.Ui_PeriodExport, QtWidgets.QDialog):
    dlgdata = QtCore.pyqtSignal(object)

    def __init__(self, begin, end, patient="", directory="", parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)

        self.dircache = directory

        self.lblPeriodStart.setText(str(begin))
        self.lblPeriodStop.setText(str(end))
        self.txtPatient.setText(patient)

        self.btnOk.clicked.connect(self.accept)
        self.btnCancel.clicked.connect(self.reject)
        self.btnBrowse.clicked.connect(self.selectFile)

    def selectFile(self):
        filepath = QtGui.QFileDialog.getSaveFileName(
            caption="Export to",
            filter="CSV files (*.csv *.dat)",
            options=QtGui.QFileDialog.DontConfirmOverwrite,
            directory=self.dircache,
        )
        # PyQt5 API change
        if type(filepath) is not str:
            filepath = filepath[0]

        if filepath:
            self.txtFile.setText(filepath)
            self.dircache = os.path.dirname(filepath)

    def accept(self):
        patient = self.txtPatient.text()
        comment = self.txtComment.text()
        periodname = self.txtPeriod.currentText()
        filepath = self.txtFile.text()
        result = (patient, comment, periodname, filepath)
        self.dlgdata.emit(result)
        super().accept()


class DlgCurveSelection(ui.Ui_CurveSelection, QtWidgets.QDialog):
    dlgdata = QtCore.pyqtSignal(object)

    def __init__(self, visible=[], hidden=[], parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)

        self.curveproperties = {}

        self.okButton.clicked.connect(self.accept)
        self.cancelButton.clicked.connect(self.reject)
        self.btnProperties.clicked.connect(self.openProperties)

        hiddenhash = {curve.name(): curve for curve in hidden}
        self.curvehash = {curve.name(): curve for curve in visible}
        self.curvehash.update(hiddenhash)

        for curve in visible:
            self.addCurve(curve.name(), checked=True)
        for curve in hidden:
            self.addCurve(curve.name(), checked=False)

    def openProperties(self):
        selected = self.lstCurves.selectedItems()
        try:
            curvename = selected[0].text()
        except IndexError:
            return
        curve = self.curvehash[curvename]

        def cb(resultdict):
            self.curveproperties[curve] = resultdict

        dlg = DlgCurveProperties(curve)
        dlg.dlgdata.connect(cb)
        dlg.exec_()

    def addCurve(self, name, checked):
        item = QtGui.QListWidgetItem()
        item.setText(name)
        item.setFlags(item.flags() | QtCore.Qt.ItemIsUserCheckable)
        if checked:
            item.setCheckState(QtCore.Qt.Checked)
        else:
            item.setCheckState(QtCore.Qt.Unchecked)
        self.lstCurves.addItem(item)

    def accept(self):
        items = self.lstCurves.findItems("", QtCore.Qt.MatchContains)
        ischecked = lambda item: item.checkState() != QtCore.Qt.Unchecked
        checked = list(filter(ischecked, items))
        visible = {self.curvehash[item.text()] for item in checked}
        result = (visible, self.curveproperties)
        self.dlgdata.emit(result)
        super().accept()


class DlgCurveProperties(ui.Ui_CurveProperties, QtWidgets.QDialog):
    dlgdata = QtCore.pyqtSignal(object)

    def __init__(self, curve, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)

        self.curve = curve

        self.okButton.clicked.connect(self.ok)
        self.cancelButton.clicked.connect(self.reject)
        self.btnColor.clicked.connect(self.chooseColor)

        connect = curve.opts['connect']
        symbol = curve.opts['symbol']
        if symbol is None:
            symbol = 'None'

        pen = curve.opts['pen']
        if isinstance(pen, QtGui.QPen):
            color = pen.color()
            width = pen.width()
        else:
            color = pen
            width = 1
        self.color = color

        idx = self.cmbConnect.findText(connect, QtCore.Qt.MatchFixedString)
        self.cmbConnect.setCurrentIndex(idx)

        idx = self.cmbSymbol.findText(symbol, QtCore.Qt.MatchFixedString)
        self.cmbSymbol.setCurrentIndex(idx)

        self.cmbSymbol.setEditText(symbol)
        self.grpName.setTitle(curve.name())
        self.txtName.setText(curve.name())
        self.spnWidth.setValue(width)
        self.btnColor.setStyleSheet(f'background-color: {color.name()}')
        self.lblSamplerate.setText(str(curve.samplerate))

    def ok(self):
        symbol = self.cmbSymbol.currentText().lower()
        symbol = None if symbol == 'none' else symbol

        connect = str(self.cmbConnect.currentText().lower())
        connect = [0] if connect == 'none' else connect

        result = {
            'name': self.txtName.text(),
            'connect': connect,
            'symbol': symbol,
            'width': self.spnWidth.value(),
            'color': self.color,
        }
        self.dlgdata.emit(result)
        self.accept()

    def chooseColor(self):
        color = QtGui.QColorDialog.getColor()
        if not color.isValid():
            return
        self.color = color
        self.btnColor.setStyleSheet(f'background-color: {color.name()}')


class DlgSetDateTime(ui.Ui_SetDateTime, QtWidgets.QDialog):
    def __init__(self, parent=None, prevdatetime=None):
        super().__init__(parent=parent)
        self.setupUi(self)

        self.dlgdata = None
        self.btnOk.clicked.connect(self.ok)
        self.btnCancel.clicked.connect(self.reject)

        if prevdatetime is None:
            datetime = QtCore.QDateTime.currentDateTime()
        else:
            datetime = QtCore.QDateTime.fromMSecsSinceEpoch(prevdatetime / 1e6)
        curdate = datetime.date()
        curtime = datetime.time()
        self.widgetCalendar.setSelectedDate(curdate)
        self.edDate.setDate(curdate)
        self.edTime.setTime(curtime)

    def ok(self):
        date = self.edDate.date()
        time = self.edTime.time()
        dt = QtCore.QDateTime(date, time, QtCore.Qt.UTC)
        timestamp = dt.toMSecsSinceEpoch() * 1e6
        self.dlgdata = timestamp
        self.accept()


class DlgCurveAlgebra(QtWidgets.QDialog):
    dlgdata = QtCore.pyqtSignal(object)

    def __init__(self, parent=None, curvecorr={}):
        super().__init__(parent=parent)
        self.setupUi(curvecorr)

    def setupUi(self, curvecorr):
        vstack = QtWidgets.QVBoxLayout(self)

        self.lbl = QtWidgets.QLabel('Enter formula for new curve:')
        vstack.addWidget(self.lbl)

        self.formula = QtWidgets.QLineEdit('a^2 + log(2*b)')
        vstack.addWidget(self.formula)

        curveslbl = '\n'.join([f'{x}: {y}' for x, y in curvecorr.items()])
        self.curveletters = QtWidgets.QLabel(curveslbl)
        vstack.addWidget(self.curveletters)

        self.buttonbox = QtWidgets.QDialogButtonBox(
            QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel
        )
        vstack.addWidget(self.buttonbox)

        self.buttonbox.accepted.connect(self.accept)
        self.buttonbox.rejected.connect(self.reject)

        self.setLayout(vstack)

    def accept(self):
        result = self.formula.text()
        self.dlgdata.emit(result)
        super().accept()


class DlgListChoice(QtWidgets.QDialog):
    dlgdata = QtCore.pyqtSignal(object)

    def __init__(self, items, title='', message='', parent=None):
        super().__init__(parent=parent)
        form = QtWidgets.QFormLayout(self)
        form.addRow(QtWidgets.QLabel(message))
        self.listView = QtWidgets.QListView(self)
        form.addRow(self.listView)
        model = QtGui.QStandardItemModel(self.listView)
        self.setWindowTitle(title)
        for item in items:
            standardItem = QtGui.QStandardItem(item)
            standardItem.setCheckable(True)
            standardItem.setCheckState(QtCore.Qt.Checked)
            standardItem.setEditable(False)
            model.appendRow(standardItem)
        self.listView.setModel(model)

        buttonBox = QtWidgets.QDialogButtonBox(
            QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel,
            QtCore.Qt.Horizontal,
            self,
        )
        form.addRow(buttonBox)
        buttonBox.accepted.connect(self.accept)
        buttonBox.rejected.connect(self.reject)

    def itemsSelected(self):
        selected = []
        model = self.listView.model()
        i = 0
        while model.item(i):
            if model.item(i).checkState():
                selected.append(model.item(i).text())
            i += 1
        return selected

    def accept(self):
        result = self.itemsSelected()
        self.dlgdata.emit(result)
        super().accept()


def askUserValue(param):
    if param.request == 'time':
        value, isok = QtGui.QInputDialog.getText(None, 'Enter time', param.description)
        try:
            value = ureg.Quantity(value)
            if value.dimensionless:
                # Default to second if no unit is specified
                value = ureg.Quantity(value.magnitude, 's')
            value = value.to_base_units().magnitude
        except (DimensionalityError, UndefinedUnitError, ValueError):
            return None
    elif param.request is str:
        value, isok = QtGui.QInputDialog.getText(None, 'Enter value', param.description)
    elif param.request is int:
        value, isok = QtGui.QInputDialog.getInt(None, 'Enter value', param.description)
    elif param.request is float:
        value, isok = QtGui.QInputDialog.getDouble(
            None, 'Enter value', param.description, decimals=3
        )
    elif param.request is bool:
        request = ['Yes', 'No']
        tmpvalue, isok = QtGui.QInputDialog.getItem(
            None, 'Enter value', param.description, request, editable=False
        )
        value = tmpvalue == 'Yes'
    elif param.request is datetime:
        dlg = DlgSetDateTime()
        isok = dlg.exec_()
        value = dlg.dlgdata
    elif type(param.request) is list:
        value, isok = QtGui.QInputDialog.getItem(
            None, 'Choose value', param.description, param.request, editable=False
        )
    else:
        raise TypeError("Unknown request type")

    if isok:
        return value
    else:
        return None


def userConfirm(question: str, title: str = '') -> bool:
    if not title:
        title = question
    reply = QtGui.QMessageBox.question(
        None, title, question, QtGui.QMessageBox.Yes, QtGui.QMessageBox.No
    )
    return reply == QtGui.QMessageBox.Yes


def askFilePath(
    func,
    caption: str,
    filename: str = '',
    folder: str = '',
    filter: str = "CSV files (*.csv *.dat)",
) -> Optional[pathlib.Path]:
    default = pathlib.Path(folder) if folder else pathlib.Path.home()
    if filename:
        default = pathlib.Path(default, filename)

    fileinfo = func(caption=caption, filter=filter, directory=str(default))
    filepath = fileinfo[0]
    if not filepath:
        # Cancel pressed
        return (None, None)
    filepath = sanitize_filepath(filepath)
    filepath = pathlib.Path(filepath).resolve()
    ext = filepath.suffix[1:]
    return (filepath, ext)


askOpenFilePath = partial(askFilePath, QtGui.QFileDialog.getOpenFileName)
askSaveFilePath = partial(askFilePath, QtGui.QFileDialog.getSaveFileName)


def askDirPath(caption: str, folder: str = '') -> Optional[pathlib.Path]:
    if not folder:
        folder = str(pathlib.Path.home())

    outdirtmp = QtGui.QFileDialog.getExistingDirectory(
        caption=caption, directory=folder
    )
    if not outdirtmp:
        # Cancel pressed
        return None
    return pathlib.Path(outdirtmp).resolve()



import os
from collections import namedtuple
from enum import Enum

Filter = namedtuple('Filter', ['name', 'parameters'])
Parameter = namedtuple('Parameter', ['description', 'request'])


class FootType(Enum):
    start = 'start'
    stop = 'stop'
    diastole = 'diastole'
    systole = 'systole'
    dicrotic = 'dicrotic'


class CycleId(Enum):
    none = 'None'
    foot = 'Pressure foot'
    pressure = 'Pressure Full'
    velocity = 'Velocity'


class PlotData:
    def __init__(self, data=[], filepath="", name=None):
        self.data = data
        self.filepath = filepath
        self._name = name

    @property
    def name(self):
        if self._name is not None:
            name = self._name
        else:
            name, _ = os.path.splitext(os.path.basename(self.filepath))
        return name

    @property
    def folder(self):
        return os.path.dirname(self.filepath)



import imp
import os
import sys
from functools import partial
from itertools import cycle

import numpy as np
import pathvalidate
from pyqtgraph.Qt import QtGui, loadUiType

sanitize_filename = partial(pathvalidate.sanitize_filename, platform='auto')
sanitize_filepath = partial(pathvalidate.sanitize_filepath, platform='auto')


def Colors():
    qtcolors = [
        QtGui.QColor(0, 114, 189),
        QtGui.QColor(217, 83, 25),
        QtGui.QColor(237, 177, 32),
        QtGui.QColor(126, 47, 142),
        QtGui.QColor(119, 172, 48),
        QtGui.QColor(77, 190, 238),
        QtGui.QColor(162, 20, 47),
    ]
    return cycle(qtcolors)


curPath = os.path.dirname(os.path.abspath(__file__))
uiBasePath = os.path.join(curPath, 'ui')


def loadUiFile(uiFile):
    uiPath = os.path.join(uiBasePath, uiFile)
    # form class and Qt base class.
    return loadUiType(uiPath)


def estimateSampleRate(series):
    intervals = np.diff(series.index)
    # 1e9 to account for ns -> Hz
    fs = 1e9 / np.median(intervals)
    if fs == np.inf:
        fs = 0
    if fs > 1:
        fs = int(round(fs))
    return fs


def loadmodule():
    defaultdir = os.path.expanduser('~')
    filepath = QtGui.QFileDialog.getOpenFileName(
        caption="Import module", filter="Python files (*.py)", directory=defaultdir
    )
    if type(filepath) is not str:
        # PyQt5 API change
        filepath = filepath[0]
    if not filepath:
        # Cancel pressed
        return

    folder, filename = os.path.split(filepath)
    modulename, _ = os.path.splitext(filename)
    f, filename, description = imp.find_module(modulename, [folder])

    bcbak = sys.dont_write_bytecode
    try:
        sys.dont_write_bytecode = True
        imp.load_module('graphysio.plugin', f, filename, description)
    finally:
        sys.dont_write_bytecode = bcbak
        f.close()


def clip(vec, vrange):
    if vrange is None:
        return vec
    xmin, xmax = vrange
    cond = (vec > xmin) & (vec < xmax)
    return vec[cond]


def truncatevecs(vecs):
    # Ensure all vectors have the same length by truncating the end
    maxidx = min(map(len, vecs))
    return [vec[0:maxidx] for vec in vecs]


def getshell(ui=None):
    import IPython

    IPython.embed(ui=ui)


def displayError(errmsg):
    msgbox = QtGui.QMessageBox()
    msgbox.setWindowTitle("Error")
    msgbox.setText(str(errmsg))
    msgbox.setStandardButtons(QtGui.QMessageBox.Ok)
    msgbox.setIcon(QtGui.QMessageBox.Critical)
    msgbox.exec_()
